import os
import logging
from ortools.constraint_solver import routing_enums_pb2, pywrapcp

logger = logging.getLogger(__name__)


def create_data_model(locations):
    """Stores the data for the problem."""
    data = {}
    data["locations"] = locations
    data["num_locations"] = len(locations)
    data["depot"] = 0
    return data


def compute_euclidean_distance_matrix(locations):
    """Computes the pairwise distance matrix between locations."""
    matrix = []
    for i, location_i in enumerate(locations):
        row = []
        for j, location_j in enumerate(locations):
            if i == j:
                row.append(0)
            else:
                distance = (
                    (location_i[0] - location_j[0]) ** 2
                    + (location_i[1] - location_j[1]) ** 2
                ) ** 0.5
                row.append(distance)
        matrix.append(row)
    return matrix


def optimize_tsp(locations):
    """Optimizes the TSP problem and returns the optimized tour."""
    data = create_data_model(locations)
    distance_matrix = compute_euclidean_distance_matrix(locations)

    # Create routing model
    manager = pywrapcp.RoutingIndexManager(data["num_locations"], 1, data["depot"])
    routing = pywrapcp.RoutingModel(manager)

    transit_callback_index = routing.RegisterTransitCallback(
        lambda i, j: distance_matrix[i][j]
    )
    routing.SetArcCostEvaluatorOfAllVehicles(transit_callback_index)

    # Set search parameters
    search_parameters = pywrapcp.DefaultRoutingSearchParameters()
    search_parameters.first_solution_strategy = (
        routing_enums_pb2.FirstSolutionStrategy.PATH_CHEAPEST_ARC
    )

    # Solve the problem
    solution = routing.SolveWithParameters(search_parameters)

    logger.debug(
        f"Locations of the problem: {locations}"
        + os.linesep
        + f"The solution:{solution}"
    )

    # Extract the optimized tour
    tour = []
    index = routing.Start(0)
    while not routing.IsEnd(index):
        tour.append(data["locations"][index])
        index = solution.Value(routing.NextVar(index))
    tour.append(data["locations"][0])

    return tour



import os
import sys
import pika
import logging

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from src.messaging.inbound import process_message
from src.config import RABBITMQ_HOST, INBOUND_QUEUE

logger = logging.getLogger(__name__)

# Define the RabbitMQ connection parameters
params = pika.ConnectionParameters(host=RABBITMQ_HOST)

# Connect to RabbitMQ and set up the inbound and outbound queues
connection = pika.BlockingConnection(params)
channel = connection.channel()
channel.queue_declare(queue=INBOUND_QUEUE)

# Set up a consumer for the inbound queue
channel.basic_consume(queue=INBOUND_QUEUE, on_message_callback=process_message)

# Start consuming messages from the inbound queue
logger.debug("Start consuming messages from the inbound queue")
channel.start_consuming()



import os
import sys
import json
import unittest
from unittest import mock

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from src.messaging.inbound import process_message

OUTBOUND_QUEUE = "tsp.outbound"

class TestProcessMessage(unittest.TestCase):
    def test_process_message(self):
        # Define the input message
        input_message = {
            "locations": [[0, 0], [1, 1], [2, 2], [3, 3]],
        }
        input_message_json = json.dumps(input_message)

        # Define the expected output message
        expected_output_message = {
            "locations": [[0, 0], [1, 1], [2, 2], [3, 3]],
            "path": [[0, 0], [3, 3], [2, 2], [1, 1], [0, 0]],
        }
        expected_output_message_json = json.dumps(expected_output_message)

        # Set up the mock channel object
        mock_channel = mock.MagicMock()

        
        # Create a pika.spec.Basic.Deliver object to pass into process_message()
        method = mock.Mock()
        method.delivery_tag = 1

        # Call the function with the input message
        process_message(mock_channel, method, None, input_message_json)

        # Check that the mock channel's basic_publish method was called with the expected arguments
        mock_channel.basic_publish.assert_called_once_with(
            exchange="", routing_key=OUTBOUND_QUEUE, body=expected_output_message_json
        )

        # Check that the mock channel's basic_ack method was called with the expected argument
        mock_channel.basic_ack.assert_called_once_with(delivery_tag=1)



import json

from src.solver.tsp_solver import optimize_tsp
from src.config import OUTBOUND_QUEUE


# Define a function to handle incoming messages
def process_message(channel, method, _properties, body):
    # Convert the message payload to a Python object
    payload = json.loads(body)

    # Extract the locations from the payload
    locations = payload["locations"]

    # Solve the TSP problem
    solution = optimize_tsp(locations)

    # Prepare the message payload for the outbound queue
    outbound_payload = {
        "locations": locations,
        "path": solution,
    }

    # Convert the payload to a JSON string
    outbound_message = json.dumps(outbound_payload)

    # Publish the message to the outbound queue
    channel.basic_publish(
        exchange="", routing_key=OUTBOUND_QUEUE, body=outbound_message
    )

    # Acknowledge the message
    channel.basic_ack(delivery_tag=method.delivery_tag)



import unittest
from src.utils.distance import euclidean_distance


class TestEuclideanDistance(unittest.TestCase):

    def test_euclidean_distance(self):
        # # test with two points with same coordinates
        self.assertEqual(euclidean_distance((0, 0), (0, 0)), 0)

        # # test with two points with different coordinates
        self.assertEqual(euclidean_distance((0, 0), (3, 4)), 5)

        # test with three points
        self.assertEqual(euclidean_distance((0, 0), (3, 4), (6, 8)), 10)

        # test with four points
        self.assertEqual(euclidean_distance((0, 0), (3, 4), (6, 8), (9, 12)), 15)



import unittest
from src.solver.tsp_solver import optimize_tsp
from src.utils.distance import euclidean_distance


class TestTspSolver(unittest.TestCase):

    def test_optimize_tsp_with_3_locations(self):
        locations = [(0, 0), (3, 0), (0, 4)]
        expected_tour = [(0, 0), (0, 4), (3, 0), (0, 0)]
        actual_tour = optimize_tsp(locations)
        self.assertEqual(actual_tour, expected_tour)

    def test_optimize_tsp_with_4_locations(self):
        locations = [(0, 0), (1, 0), (2, 0), (0, 1)]
        expected_tour = [(0, 0), (0, 1), (2, 0), (1, 0), (0, 0)]
        actual_tour = optimize_tsp(locations)
        self.assertEqual(actual_tour, expected_tour)

    def test_optimize_tsp_with_5_locations(self):
        locations = [(0, 0), (1, 0), (2, 0), (2, 1), (0, 1)]
        expected_tour = [(0, 0), (0, 1), (2, 1), (2, 0), (1, 0), (0, 0)]
        actual_tour = optimize_tsp(locations)
        self.assertEqual(actual_tour, expected_tour)



import pika
import logging

from src.config import RABBITMQ_HOST, OUTBOUND_QUEUE

logger = logging.getLogger(__name__)


# Define the RabbitMQ connection parameters
params = pika.ConnectionParameters(host=RABBITMQ_HOST)

# Connect to RabbitMQ and set up the inbound and outbound queues
connection = pika.BlockingConnection(params)
channel = connection.channel()
channel.queue_declare(queue=OUTBOUND_QUEUE)


def callback(channel, method, _properties, body):
    # Receive message from outbound queue
    tour = body.decode()

    logger.debug(f"Optimized Tour: {tour}")

    # Acknowledge message received
    channel.basic_ack(delivery_tag=method.delivery_tag)


# Start consuming from outbound queue
channel.basic_consume(queue=OUTBOUND_QUEUE, on_message_callback=callback)

logger.debug("Waiting for messages in outbound")
channel.start_consuming()



from typing import List
from math import sqrt


def euclidean_distance(*locations: List[float]) -> float:
    """
    Calculates the Euclidean distance between any number of locations.
    """
    if len(locations) < 2:
        raise ValueError(
            "At least two locations are required to calculate the Euclidean distance."
        )
    distance = 0
    for i in range(len(locations) - 1):
        squared_distance = sum(
            (p1 - p2) ** 2 for p1, p2 in zip(locations[i], locations[i + 1])
        )
        distance += sqrt(squared_distance)
    return distance



from setuptools import setup, find_packages

setup(
    name='tsp-planner',
    version='0.2',
    packages=find_packages(),
    install_requires=[
        'pika',
        'ortools'
    ],
    entry_points={
        'console_scripts': [
            'tsp_planner = tsp_planner.main:main'
        ]
    },
    author='Planning Tech',
    description='A service for solving the Travelling Salesman Problem using OR-tools library',
    url='https://github.com/ghorbani-mohammad/travelling-salesman-problem'
)



import os
import logging
from dotenv import load_dotenv

load_dotenv()

RABBITMQ_HOST = os.getenv("RABBITMQ_HOST")
INBOUND_QUEUE = os.getenv("INBOUND_QUEUE")
OUTBOUND_QUEUE = os.getenv("OUTBOUND_QUEUE")

# Set up logging
logging.basicConfig(
    level=os.getenv("LOG_LEVEL", "INFO"),
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    filename="app.log",
)



import os
import sys
import unittest
from io import StringIO
from unittest.mock import MagicMock

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from src.messaging.outbound import callback


class TestCallback(unittest.TestCase):
    def test_callback(self):
        # Create mock objects for the channel and the method
        channel_mock = MagicMock()
        method_mock = MagicMock(delivery_tag='mock_delivery_tag')

        # Create a mock message body
        message_body = b'Test message'

        # Redirect the standard output to a string buffer
        captured_output = StringIO()
        sys.stdout = captured_output

        # Call the callback function with the mock objects
        callback(channel_mock, method_mock, None, message_body)

        # Restore the standard output
        sys.stdout = sys.__stdout__

        # Get the printed output as a string
        print_calls = captured_output.getvalue()

        # Assert that the message body was decoded and printed correctly
        channel_mock.basic_ack.assert_called_once_with(delivery_tag='mock_delivery_tag')
        self.assertEqual(channel_mock.basic_ack.call_count, 1)

        # Check that the printed message contains the expected string
        self.assertIn('Optimized Tour:', print_calls)

        # Check that the printed message contains the decoded message body
        self.assertIn(message_body.decode(), print_calls)  
